package net.wanji.utc.common;

import java.io.Serializable;

import static net.wanji.utc.common.ResultEnum.INTERNAL_SERVER_ERROR;
import static net.wanji.utc.common.ResultEnum.SUCCESS;

/**
 * @author wuxiaokai
 * @date 2022/11/15 13:13:36
 */
public class Result<T> implements Serializable {
    private static final long serialVersionUID = -8491397399858539755L;

    private Integer state;
    private String message;
    private T content;

    public Integer getState() {
        return state;
    }

    public void setState(Integer state) {
        this.state = state;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public T getContent() {
        return content;
    }

    public void setContent(T content) {
        this.content = content;
    }

    public Result() {
    }

    public Result(Integer state, String message, T content) {
        this.state = state;
        this.message = message;
        this.content = content;
    }

    public static <T> Result<T> response(Integer state, String message) {
        return response(state, message, null);
    }

    public static <T> Result<T> response(Integer state, String message, T content) {
        return new Result<>(state, message, content);
    }

    public static <T> Result<T> success() {
        return Result.response(SUCCESS.getResultCode(), SUCCESS.getResultMsg(), null);
    }

    public static <T> Result<T> success(T content) {
        return Result.response(SUCCESS.getResultCode(), SUCCESS.getResultMsg(), content);
    }

    public static <T> Result<T> success(String message) {
        return Result.response(SUCCESS.getResultCode(), message, null);
    }

    public static <T> Result<T> error(Integer state, String message, T content) {
        return Result.response(state, message, content);
    }

    public static <T> Result<T> error(Integer state, String message) {
        return Result.response(state, message, null);
    }

    public static <T> Result<T> error(Integer state) {
        return Result.response(state, INTERNAL_SERVER_ERROR.getResultMsg(), null);
    }

    public static <T> Result<T> error() {
        return error(INTERNAL_SERVER_ERROR);
    }

    public static <T> Result<T> error(ResultEnum resultEnum, String message) {
        return Result.response(resultEnum.getResultCode(), message, null);
    }

    public static <T> Result<T> error(ResultEnum resultEnum) {
        return Result.response(resultEnum.getResultCode(), resultEnum.getResultMsg(), null);
    }

    public static <T> Result<T> error(String message) {
        return Result.response(INTERNAL_SERVER_ERROR.getResultCode(), message, null);
    }
}
