package net.wanji.utc.controller;

/**
 * @author Kent HAN
 * @date 2022/12/1 15:07
 */

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.RequiredArgsConstructor;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.databus.vo.LightsStatusVO2;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.service.runninginfo.SignalStatusService;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.core.MediaType;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * 运行状态、告警、灯态信息接口
 *
 * @date 2022/11/15 9:38
 */
@Api(value = "运行状态、告警、灯态信息接口", description = "运行状态、告警、灯态信息接口")
@RequestMapping("/signalStatus")
@RestController
@RequiredArgsConstructor
public class SignalStatusController {
    private final SignalStatusService signalStatusService;

    @AspectLog(description = "运行状态、告警数据", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/runningStatusAlarm", produces = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "运行状态、告警数据", notes = "运行状态、告警数据", response = SignalStatusLogPO.class,
            produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SignalStatusLogPO.class)
    })
    public JsonViewObject runningStatusAlarm() throws Exception {
        List<SignalStatusLogPO> signalStatusLogPOList = signalStatusService.runningStatusAlarm();
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(signalStatusLogPOList);

    }

    @AspectLog(description = "灯态数据", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/lightStatus", produces = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "灯态数据", notes = "灯态数据", response = LightsStatusVO.class,
            produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = LightsStatusVO.class)
    })
    public JsonViewObject lightStatus() throws Exception {
        List<LightsStatusVO> lightsStatusVOList = signalStatusService.lightStatus();
        return JsonViewObject.newInstance().success(lightsStatusVOList);
    }

    @AspectLog(description = "灯态数据V2", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/lightStatusV2", produces = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "灯态数据V2", notes = "灯态数据V2", response = LightsStatusVO.class,
            produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = LightsStatusVO.class)
    })
    public JsonViewObject lightStatusV2() throws Exception {
        //List<LightsStatusVO> lightsStatusVOList = signalStatusService.lightStatus4StaticScheme();
        List<LightsStatusVO> lightsStatusVOList = signalStatusService.lightStatus();
        List<LightsStatusVO2> listResult = new ArrayList<>(lightsStatusVOList.size());

        for (LightsStatusVO lightsStatusVO : lightsStatusVOList) {
            Integer cyclePhaseCountDown = lightsStatusVO.getCyclePhaseCountDown();
            LightsStatusVO2 lightsStatusVO2 = new LightsStatusVO2();
            BeanUtils.copyProperties(lightsStatusVO, lightsStatusVO2);
            Map<String, Object> dirLampGroupMap = lightsStatusVO.getDirLampGroupMap();
            List<LightsStatusVO2.DirInfo> dirList = new ArrayList<>();
            Map<String, Object> phaseMap = lightsStatusVO.getPhaseMap();
            for (Map.Entry<String, Object> groupEntry : dirLampGroupMap.entrySet()) {
                LightsStatusVO2.DirInfo dirInfo = new LightsStatusVO2.DirInfo();
                String dir = groupEntry.getKey();
                Map<String, String> turnColorMap = (Map<String, String>) groupEntry.getValue();
                List<LightsStatusVO2.TurnInfo> turnList = new ArrayList<>();
                for (Map.Entry<String, String> turnEntry : turnColorMap.entrySet()) {
                    String turn = turnEntry.getKey();
                    String color = turnEntry.getValue();
                    LightsStatusVO2.TurnInfo turnInfo = new LightsStatusVO2.TurnInfo();
                    turnInfo.setTurn(String.valueOf(turn));
                    turnInfo.setColor(color);
                    String key = dir.concat(Constants.SEPARATOR_UNDER_LINE).concat(turn);
                    Object countDown = phaseMap.get(key);
                    if (Objects.isNull(countDown)) {
                        countDown = cyclePhaseCountDown;
                    }
                    if (Objects.isNull(cyclePhaseCountDown)) {
                        countDown = lightsStatusVO.getCycleCountDown();
                    }
                    turnInfo.setCountDown((int) countDown);
                    turnList.add(turnInfo);
                }
                dirInfo.setDir(dir);
                dirInfo.setTurnList(turnList);
                dirList.add(dirInfo);
            }
            lightsStatusVO2.setDirLampGroupMapList(dirList);
            listResult.add(lightsStatusVO2);
        }
        return JsonViewObject.newInstance().success(listResult);
    }

    @AspectLog(description = "灯态数据历史数据", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/lightStatusHist", produces = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "灯态数据历史数据", notes = "灯态数据历史数据", response = LightsStatusVO2.class,
            produces = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = LightsStatusVO2.class)
    })
    public JsonViewObject lightStatusHist(String crossId, Integer startBatchTime, Integer endBatchTime) throws Exception {
        List<LightsStatusVO2> lightsStatusVOList = signalStatusService.lightStatusHist(crossId, startBatchTime, endBatchTime);
        return JsonViewObject.newInstance().success(lightsStatusVOList);
    }
}
