package net.wanji.utc.service.control.impl;

import lombok.RequiredArgsConstructor;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.vo.*;
import net.wanji.feign.service.common.FeignCommon;
import net.wanji.utc.cache.CrossInfoCache;
import net.wanji.utc.cache.UtcFeignClientCache;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.po.hk.request.DelBaseConfigPO;
import net.wanji.utc.service.control.ControlCommandService;
import net.wanji.utc.util.HttpRestUtil;
import net.wanji.utc.util.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/01/12 13:18
 */
@Service("wanJiControlCommandService")
@RequiredArgsConstructor
public class WanJiControlCommandServiceImpl implements ControlCommandService {

    private final HttpRestUtil httpRestUtil;
    @Value("url")
    private String rpcUri;
    private final UtcFeignClientCache utcFeignClientCache;
    private final CrossInfoCache crossInfoCache;

    /**
     * @param schemeSendVO
     * @return
     * @throws Exception
     */
    @Override
    public JsonViewObject schemeSend(SchemeSendVO schemeSendVO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(schemeSendVO.getCrossCode());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).schemeSend(schemeSendVO);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议下发方案成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject planSend(PlanSendVO planSendVO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(planSendVO.getCrossCode());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).planSend(planSendVO);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议下发计划成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject scheduleSend(ScheduleSendVO scheduleSendVO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(scheduleSendVO.getCrossCode());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).scheduleSend(scheduleSendVO);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议下发时间表成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject lockControl(ControlCommandVO commandVO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(commandVO.getCrossCode());
        LockControlVO lockControlVO = new LockControlVO();
        lockControlVO.setCrossCode(commandVO.getCrossCode());
        lockControlVO.setCommand(commandVO.getCommand());
        lockControlVO.setDuration(commandVO.getDuration());
        lockControlVO.setPhaseList(commandVO.getPhaseList());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).lockControl(lockControlVO);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议锁定控制成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject stepControl(String crossId, Integer command, Integer stepNum) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).stepControl(crossId, command, stepNum);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议步进控制成功！");
        }
        return jsonViewObject;
    }

    /**
     * 全红黄闪关灯
     *
     * @param crossId     信号机编号
     * @param command     指令 1 开 0 关
     * @param commandType 命令类型
     * @return
     * @throws Exception
     */
    @Override
    public JsonViewObject setSignalControl(String crossId, Integer command, Integer commandType) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        FeignCommon utcFeignClientService = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        ControlCommandVO controlCommandVO = new ControlCommandVO();
        controlCommandVO.setCrossCode(crossId);
        controlCommandVO.setCommand(command);
        controlCommandVO.setDuration(999);
        if (Objects.equals(Constants.CLOSE_LIGHT_CONTROL, commandType)) {
            controlCommandVO.setControlType(Constants.CLOSE_LIGHT_CONTROL);
            jsonViewObject = utcFeignClientService.closeLightControl(controlCommandVO);
        }
        if (Objects.equals(Constants.YELLOW_LIGHT_CONTROL, commandType)) {
            controlCommandVO.setControlType(Constants.YELLOW_LIGHT_CONTROL);
            jsonViewObject = utcFeignClientService.closeLightControl(controlCommandVO);
        }
        if (Objects.equals(Constants.ALL_RED_CONTROL, commandType)) {
            controlCommandVO.setControlType(Constants.ALL_RED_CONTROL);
            jsonViewObject = utcFeignClientService.closeLightControl(controlCommandVO);
        }
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议设置信号机控制模式成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject recoverSchedule(String crossId) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).recoverSchedule(crossId);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议恢复时间表成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject phaseTimingSend(PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(phaseTimingSendVO.getCrossCode());
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).recoverSchedule(phaseTimingSendVO.getCrossCode());
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议下发相位配时成功！");
        }
        return jsonViewObject;
    }

    @Override
    public JsonViewObject phaseDiffSend() {
        return null;
    }

    @Override
    public JsonViewObject delBaseConfig(DelBaseConfigPO delBaseConfigPO) {
        return null;
    }

    @Override
    public JsonViewObject timing(String crossCode, Date date) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossCode);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).signalTiming(crossCode, date);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议校时成功!");
        }
        return jsonViewObject.fail("万集标准协议校时失败!");
    }

    @Override
    public JsonViewObject tempScheme(String crossCode, Integer schemeNo) throws Exception {
        String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossCode);
        JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).tempScheme(crossCode, schemeNo);
        if (StringUtils.endsWithIgnoreCase("success", jsonViewObject.getStatus())) {
            return jsonViewObject.success("万集标准协议下发临时方案成功!");
        }
        return jsonViewObject.fail("万集标准协议校时失败!");
    }
}
