package net.wanji.utc.service.runninginfo.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.BaseCrossInfo;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.service.runninginfo.HkRunningStatusService;
import net.wanji.utc.util.PathUtil;
import net.wanji.utc.util.RedisUtil;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author Kent HAN
 * @date 2022/11/21 10:05
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class HkRunningStatusServiceImpl implements HkRunningStatusService {
    private final ArtemisConfig artemisConfig;
    private final RedisUtil redisUtil;

    @Override
    public List<SignalStatusLogPO> getHkRunningStatus(List<BaseCrossInfo> baseCrossInfoList) throws Exception {
        List<SignalStatusLogPO> result = new ArrayList<>();
        SignalStatusLogPO po;
        try {
            Map<String, String> runningPath = PathUtil.getPathMapByApiCode("getWorkingStateBatch");
            for (BaseCrossInfo baseCrossInfo : baseCrossInfoList) {
                JSONArray jsonBody = new JSONArray();
                jsonBody.add(baseCrossInfo.getCode());
                String strResult = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, runningPath, jsonBody.toJSONString(),
                        null, null, "application/json", null);
                JSONObject object = JSON.parseObject(strResult);
                Object o = object.get("code");
                // 多信号机情况下，跳过返回信息不成功的获取返回消息成功的信号机数据
                if (!Objects.equals(String.valueOf(o), "0")) {
                    continue;
                }
                if (Constants.HK_SUCCESS_CODE.equals(object.getInteger(Constants.HK_CODE_KEY))) {
                    JSONArray data = object.getJSONArray("data");
                    for (int i = 0; i < data.size(); i++) {
                        JSONObject obj =  data.getJSONObject(i);
                        po = new SignalStatusLogPO();
                        po.setCrossId(baseCrossInfo.getCrossId());
                        po.setSignalId(baseCrossInfo.getCode());
                        po.setStatus(transferStatus(obj.getInteger("state")));
                        // 告警信息
                        Integer hasFault = obj.getInteger("hasFault");
//                        if (hasFault == 0) {
                            po.setFaultType(hasFault);
//                        }
                        Map<String, String> controlModeMap = Constants.controlModeMap;
                        Map<String, Integer> runModeControlMap = Constants.runModeControlMap;
                        if (!runModeControlMap.isEmpty()) {
                            String currentControl = null;
                            if (controlModeMap.isEmpty()) {
                                currentControl = "11";
                            } else {
                                currentControl = controlModeMap.get(baseCrossInfo.getCrossId());
                            }
                            po.setControlType(runModeControlMap.get(currentControl));
                        }
                        result.add(po);
                    }
                }
            }
        } catch(Exception e){
            log.error("调用海康平台路口运行状态异常：", e);
            throw new Exception(e);
        }
        return result;
    }

    /**
     * 转换海康信号机运行状态，0 离线 1 在线
     * @param state  海康信号机状态定义 0 在线 1 离线
     * @return
     */
    private int transferStatus(int state) {
        return 0 == state ? 1 : 0;
    }
}
