package net.wanji.utc.service.runninginfo.impl;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.dao.entity.CrossLightsStatusHistPO;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.dao.mapper.CrossLightsStatusHistMapper;
import net.wanji.databus.dao.mapper.ManufacturerInfoMapper;
import net.wanji.databus.dao.mapper.SignalStatusLogMapper;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.ManufacturerInfoPO;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.BaseCrossInfo;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.databus.vo.LightsStatusVO2;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.constant.RedisKeyConst;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.service.runninginfo.HkLightsStatusService;
import net.wanji.utc.service.runninginfo.HkRunningStatusService;
import net.wanji.utc.service.runninginfo.SignalStatusService;
import net.wanji.utc.service.runninginfo.WanJiCommonRunningStatusService;
import net.wanji.utc.task.SignalStatus4StaticSchemeTask;
import net.wanji.utc.util.ListUtil;
import net.wanji.utc.util.RedisUtil;
import net.wanji.utc.util.StringUtils;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.MatchQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @date 2022/11/21 9:11
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class SignalStatusServiceImpl implements SignalStatusService {
    private final ManufacturerInfoMapper manufacturerInfoMapper;
    private final CrossInfoMapper crossInfoMapper;
    private final HkRunningStatusService hkRunningStatusService;
    private final HkLightsStatusService hkLightsStatusService;
    private final RedisUtil redisUtil;
    private final SignalStatusLogMapper signalStatusLogMapper;
    private final WanJiCommonRunningStatusService wanJiRunningStatusService;
    private final CrossLightsStatusHistMapper crossLightsStatusHistMapper;
    @Resource
    RestHighLevelClient client;

    @Override
    @Transactional
    public List<SignalStatusLogPO> runningStatusAlarm() throws Exception {
        List<SignalStatusLogPO> resList = new ArrayList<>();
        try {
            BasicEnum.ManufacturerEnum[] manufacturerEnums = BasicEnum.ManufacturerEnum.values();
            for (BasicEnum.ManufacturerEnum manufacturerEnum : manufacturerEnums) {
                List<BaseCrossInfo> baseCrossInfoList = getBaseCrossInfoList(manufacturerEnum);
                if (baseCrossInfoList == null) {
                    continue;
                }
                List<SignalStatusLogPO> signalStatusLogPOList = new ArrayList<>();
                if (manufacturerEnum.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
                    // 海康
                    // 3.2.5信号机运行状态和告警信息
                    resList.addAll(hkRunningStatusService.getHkRunningStatus(baseCrossInfoList));
                } else if (manufacturerEnum.getCode().equals(BasicEnum.ManufacturerEnum.DT.getCode())) {
                    // todo 其他厂商
                    List<String> crossIdList = baseCrossInfoList.stream().map(BaseCrossInfo::getCrossId).collect(Collectors.toList());
                    resList.addAll(wanJiRunningStatusService.runningStatus(crossIdList));
                }
                // 更新Redis
                if (null != signalStatusLogPOList) {
                    for (SignalStatusLogPO signalStatusLogPO : signalStatusLogPOList) {
                        String field = manufacturerEnum.getCode() + Constants.SEPARATOR_UNDER_LINE +
                                signalStatusLogPO.getSignalId();
                        redisUtil.setHash(RedisKeyConst.KEY_PREFIX + RedisKeyConst.TELESEME_STATUS, field,
                                signalStatusLogPO.getStatus() + "");
                    }
                    // 更新日志数据库
                    if (ListUtil.isNotEmpty(signalStatusLogPOList)) {
                        signalStatusLogMapper.insertBatch(signalStatusLogPOList);
                        resList.addAll(signalStatusLogPOList);
                    }
                }
            }
        } catch (Exception e) {
            log.error("运行状态、告警服务异常", e);
            throw new Exception(e);
        }
        return resList;
    }

    @Override
    public List<SignalStatusLogPO> runningStatusAlarm(String crossId) {
        List<SignalStatusLogPO> signalStatusLogPOList = new ArrayList<>();
        return signalStatusLogPOList;
    }

    @Override
    public List<LightsStatusVO> lightStatus(String crossId) throws Exception {
        List<LightsStatusVO> lightsStatusVOList = new ArrayList<>();
        String manufacturerCode = getManufacturerCode(crossId);
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            // 海康
            lightsStatusVOList.addAll(hkLightsStatusService.getHkLightsStatus(crossId));
        } else if (StringUtils.equals("STATIC", manufacturerCode)) {
            LightsStatusVO lightsStatusVO = SignalStatus4StaticSchemeTask.currentCrossLightStatusCache.get(crossId);
            lightsStatusVOList.add(lightsStatusVO);
        } else {
            // 默认万集
            lightsStatusVOList.addAll(wanJiRunningStatusService.lightsStatus(Arrays.asList(crossId)));
        }
        return lightsStatusVOList;
    }

    private String getManufacturerCode(String crossId) {
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        if (Objects.nonNull(crossInfoPO)) {
            Integer manufacturerId = crossInfoPO.getManufacturerId();
            ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerId);
            if (Objects.nonNull(manufacturerInfoPO)) {
                return manufacturerInfoPO.getCode();
            }
        }
        return "WJ";
    }

    @Override
    @Transactional
    public List<LightsStatusVO> lightStatus() throws Exception {
        List<LightsStatusVO> resList = new ArrayList<>();
        BasicEnum.ManufacturerEnum[] manufacturerEnums = BasicEnum.ManufacturerEnum.values();
        for (BasicEnum.ManufacturerEnum manufacturerEnum : manufacturerEnums) {
            List<BaseCrossInfo> baseCrossInfoList = getBaseCrossInfoList(manufacturerEnum);
            if (baseCrossInfoList == null) {
                continue;
            }
            List<LightsStatusVO> lightsStatusVOList = null;
            if (manufacturerEnum.getCode().equals(BasicEnum.ManufacturerEnum.HK.getCode())) {
                // 海康
                // 3.2.4灯态
                lightsStatusVOList = hkLightsStatusService.getHkLightsStatus(baseCrossInfoList);
            } else if (StringUtils.equals("STATIC", manufacturerEnum.getCode())) {
                // 静态方案
                Map<String, LightsStatusVO> currentCrossLightStatusCache = SignalStatus4StaticSchemeTask.currentCrossLightStatusCache;
                if (!currentCrossLightStatusCache.isEmpty()) {
                    for (Map.Entry<String, LightsStatusVO> entry : currentCrossLightStatusCache.entrySet()) {
                        LightsStatusVO value = entry.getValue();
                        lightsStatusVOList.add(value);
                    }
                }
            } else {
                // todo 其他厂商
                List<String> crossIdList = baseCrossInfoList.stream().map(BaseCrossInfo::getCrossId).collect(Collectors.toList());
                lightsStatusVOList = wanJiRunningStatusService.lightsStatus(crossIdList);
            }
            // 更新Redis
            if (lightsStatusVOList != null) {
                for (LightsStatusVO lightsStatusVO : lightsStatusVOList) {
                    if (Objects.nonNull(lightsStatusVO)) {
                        String field = manufacturerEnum.getCode() + Constants.SEPARATOR_UNDER_LINE +
                                lightsStatusVO.getCode();
                        redisUtil.setHash(RedisKeyConst.KEY_PREFIX + RedisKeyConst.LIGHTS_STATUS, field,
                                JSON.toJSONString(lightsStatusVO));
                    }
                }
                resList.addAll(lightsStatusVOList);
            }
        }
        return resList;
    }

    @Override
    public List<LightsStatusVO> lightStatus4StaticScheme() throws Exception {
        Map<String, LightsStatusVO> runningStateInfoCache = SignalStatus4StaticSchemeTask.runningStateInfoCache;
        List<LightsStatusVO> result = new ArrayList<>();
        for (Map.Entry<String, LightsStatusVO> entry : runningStateInfoCache.entrySet()) {
            result.add(entry.getValue());
        }
        return result;
    }

    private List<BaseCrossInfo> getBaseCrossInfoList(BasicEnum.ManufacturerEnum manufacturerEnum) {
        // 查询路口信息
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectByCode(manufacturerEnum.getCode());
        Integer manufacturerId = manufacturerInfoPO.getId();
        List<CrossInfoPO> crossInfoPOList = crossInfoMapper.selectByManufacturerId(manufacturerId);
        if (ListUtil.isEmpty(crossInfoPOList)) {
            return null;
        }
        // 信号机接口入参
        List<BaseCrossInfo> baseCrossInfoList = new ArrayList<>();
        for (CrossInfoPO infoPO : crossInfoPOList) {
            BaseCrossInfo baseCrossInfo = new BaseCrossInfo();
            baseCrossInfo.setCode(infoPO.getCode());
            baseCrossInfo.setCrossId(infoPO.getId());
            baseCrossInfo.setManufacturerCode(manufacturerEnum.getCode());
            baseCrossInfoList.add(baseCrossInfo);
        }
        return baseCrossInfoList;
    }

    @Override
    public List<LightsStatusVO2> lightStatusHist(String crossId, Integer startBatchTime, Integer endBatchTime) throws Exception {
        if (StringUtils.equals("es", "")) {
            return fromElastic(crossId, startBatchTime, endBatchTime);
        }
        return fromMysql(crossId, startBatchTime, endBatchTime);
    }

    private List<LightsStatusVO2> fromMysql(String crossId, Integer startBatchTime, Integer endBatchTime) throws JsonProcessingException {
        List<LightsStatusVO2> result = new ArrayList<>();
        List<CrossLightsStatusHistPO> crossLightsStatusHistPOS = crossLightsStatusHistMapper.selectRangeList(crossId, startBatchTime, endBatchTime);
        if (!CollectionUtils.isEmpty(crossLightsStatusHistPOS)) {
            for (CrossLightsStatusHistPO crossLightsStatusHistPO : crossLightsStatusHistPOS) {
                String lightsStatusJson = crossLightsStatusHistPO.getLightsStatusJson();
                ObjectMapper instance = JacksonUtils.getInstance();
                List<LightsStatusVO2> lightsStatusVOS = instance.readValue(lightsStatusJson, new TypeReference<List<LightsStatusVO2>>() {
                });
                if (!CollectionUtils.isEmpty(lightsStatusVOS)) {
                    LightsStatusVO2 lightsStatusVO = lightsStatusVOS.get(0);
                    result.add(lightsStatusVO);
                }
            }
        }
        return result;
    }

    private List<LightsStatusVO2> fromElastic(String crossId, Integer startBatchTime, Integer endBatchTime) throws Exception {
        try {
            List<LightsStatusVO2> result = new ArrayList<>();
            SearchRequest searchRequest = new SearchRequest(Constants.SIGNAL_LIGHT_HISTORY_KEY);
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();

            MatchQueryBuilder matchQuery = QueryBuilders.matchQuery("crossId", crossId);
            RangeQueryBuilder rangeQuery = QueryBuilders.rangeQuery("timeStamp")
                    .gte(startBatchTime)
                    .lte(endBatchTime);

            BoolQueryBuilder boolQuery = QueryBuilders.boolQuery()
                    .must(matchQuery)
                    .must(rangeQuery);

            searchSourceBuilder
                    .query(boolQuery);
            //.size(10000);
            searchRequest.source(searchSourceBuilder);

            SearchResponse response = client.search(searchRequest, RequestOptions.DEFAULT);
            SearchHits hits = response.getHits();

            for (SearchHit hit : hits) {
                String sourceAsString = hit.getSourceAsString();
                LightsStatusVO2 lightsStatusVO2 =
                        JacksonUtils.getInstance().readValue(sourceAsString, LightsStatusVO2.class);
                result.add(lightsStatusVO2);
            }

            return result;
        } catch (Exception e) {
            log.error("elastic查询历史灯态异常", e);
            throw new Exception(e);
        }
    }
}
