package net.wanji.utc.service.runninginfo.impl;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.po.SignalStatusLogPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.cache.CrossInfoCache;
import net.wanji.utc.cache.UtcFeignClientCache;
import net.wanji.utc.service.runninginfo.WanJiCommonRunningStatusService;
import net.wanji.utc.util.HttpRestUtil;
import net.wanji.utc.util.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author duanruiming
 * @date 2023/01/11 17:34
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class WanJiCommonRunningStatusServiceImpl implements WanJiCommonRunningStatusService {

    private final HttpRestUtil httpRestUtil;
    @Value("url")
    private String rpcUri;
    private final CrossInfoCache crossInfoCache;
    private final UtcFeignClientCache utcFeignClientCache;

    @Override
    public List<SignalStatusLogPO> runningStatus(List<String> crossIdList) throws Exception {
        List<SignalStatusLogPO> result = new ArrayList<>();
        for (String crossId : crossIdList) {
            String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
            if (StringUtils.isEmpty(manufacturerIdCode)) {
                continue;
            }
            JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).runningStatusAlarm(crossId);
            if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
                throw new Exception("获取运行状态、告警数据远程服务调用异常，异常信息" + jsonViewObject.getMessage());
            }
            ObjectMapper mapper = JacksonUtils.getInstance();
            result.addAll(mapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<SignalStatusLogPO>>() {}));
        }

        return result;
    }

    @Override
    public List<LightsStatusVO> lightsStatus(List<String> crossIdList) throws Exception {
        Set<String> set = new HashSet<>();
        List<LightsStatusVO> result = new ArrayList<>();
        for (String crossId : crossIdList) {
            String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
            set.add(manufacturerIdCode);
            JsonViewObject jsonViewObject = utcFeignClientCache.getUtcFeignClientService(manufacturerIdCode).lightStatus(crossId);
            if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
                throw new Exception("获取灯态数据远程服务调用异常，异常信息" + jsonViewObject.getMessage());
            }
            ObjectMapper objectMapper = JacksonUtils.getInstance();
            result.addAll(objectMapper.convertValue(jsonViewObject.getContent(), new TypeReference<List<LightsStatusVO>>() {}));
        }
        return result;
    }

}
