package net.wanji.utc.service.staticinfo.impl;

import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.ManufacturerInfoMapper;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.po.ManufacturerInfoPO;
import net.wanji.utc.common.constant.Constants;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.service.staticinfo.HkCrossInfoService;
import net.wanji.utc.util.PathUtil;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @date 2022/11/15 13:18
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class HkCrossInfoServiceImpl implements HkCrossInfoService {
    private final ArtemisConfig artemisConfig;
    private final ManufacturerInfoMapper manufacturerInfoMapper;
    private final List<CrossInfoPO> res = new ArrayList<>();

    @Override
    public List<CrossInfoPO> hkCrossBasicInfo() throws Exception {
        String apiName = "crossBasicInfo";
        JSONObject responseObj = getHkJsonObjectByPage(apiName);
        if(!Constants.HK_SUCCESS_CODE.equals(responseObj.getInteger(Constants.HK_CODE_KEY))) {
            return res;
        }
        JSONObject data = responseObj.getJSONObject("data");
        JSONArray list = data.getJSONArray("list");
        list.stream()
                .map(signalObj -> (JSONObject)signalObj)
                .forEach(this::consumeSignalObj);
        return res;
    }

    @Override
    public void clearRes() {
        res.clear();
    }

    private void consumeSignalObj(JSONObject signalObj) {
        String crossCode = signalObj.getString("crossCode");
        String crossName = signalObj.getString("crossName");
        CrossInfoPO crossInfoPO = new CrossInfoPO();
        // 32位随机ID
        crossInfoPO.setId(IdUtil.simpleUUID());
        crossInfoPO.setName(crossName);
        crossInfoPO.setCode(crossCode);
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectByCode(
                BasicEnum.ManufacturerEnum.HK.getCode());
        if (manufacturerInfoPO != null) {
            crossInfoPO.setManufacturerId(manufacturerInfoPO.getId());
        }
        crossInfoPO.setIp(signalObj.getString("ip"));
        crossInfoPO.setPort(signalObj.getInteger("port"));
        // 经纬度
        String longitude = signalObj.getString("longitude");
        String latitude = signalObj.getString("latitude");
        crossInfoPO.setLocation(longitude + "," + latitude);
        crossInfoPO.setVersion(signalObj.getString("sigVersion"));
        crossInfoPO.setInstallTime(signalObj.getDate("installTime"));

        res.add(crossInfoPO);
    }

    private JSONObject getHkJsonObjectByPage(String apiName) throws Exception {
        // key-"http://"，value-"/artemis/api/itscms-scms/v1/crossinfo/hkCrossBasicInfo"
        Map<String, String> path = PathUtil.getPathMapByApiCode(apiName);
        JSONObject requestContent = new JSONObject();
        // 海康固定入参字段名
        requestContent.put("pageNo", 1);
        requestContent.put("pageSize", 9999);
        requestContent.put("searchObj", new JSONObject());
        String body = requestContent.toJSONString();
        // 请求参数
        String responseStr = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, body, null,
                null, "application/json", null);
        JSONObject responseObj = JSON.parseObject(responseStr);
        return responseObj;
    }
}
