package net.wanji.utc.service.staticinfo.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import net.wanji.common.enums.DateStyle;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.databus.dao.mapper.CrossInfoMapper;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.vo.DetailCrossInfoVO;
import net.wanji.utc.common.commonentity.HttpRequest;
import net.wanji.utc.common.genericentity.ManufacturerRes;
import net.wanji.utc.po.hk.response.HKResponse;
import net.wanji.utc.po.hk.response.SignalTimePO;
import net.wanji.utc.service.staticinfo.OthersStaticInfoService;
import net.wanji.utc.util.HttpRestUtil;
import net.wanji.utc.util.PathUtil;
import net.wanji.utc.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;

import static net.wanji.utc.common.constant.Constants.HK_SUCCESS_STRING_CODE;

/**
 * @author Kent HAN
 * @date 2022/11/16 11:03
 */
@Service
public class OthersStaticInfoServiceImpl implements OthersStaticInfoService {
    @Autowired
    private HttpRestUtil httpRestUtil;
    @Autowired
    private ArtemisConfig artemisConfig;
    @Autowired
    private CrossInfoMapper crossInfoMapper;


    @Override
    public ManufacturerRes<DetailCrossInfoVO> crossBasicInfo(String manufacturerCode) {
        HttpRequest httpRequest = new HttpRequest(manufacturerCode, "queryCrossing");
        String result = httpRestUtil.doExecute(httpRequest.getUrl(), httpRequest.getHeaders(), HttpMethod.POST, null);
        if (StringUtils.isEmpty(result)) {
            return null;
        }
        return JSON.parseObject(result, new TypeReference<ManufacturerRes<DetailCrossInfoVO>>() {});
    }

    @Override
    public JsonViewObject signalTime(String crossId) throws Exception {
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        Map<String, String> path = PathUtil.getPathMapByApiCode("getSignalSysTime");
        String requestBody = JacksonUtils.getInstance().writeValueAsString(Arrays.asList(crossInfoPO.getCode()));
        String responseStr = ArtemisHttpUtil.doPostStringArtemis(artemisConfig, path, requestBody, null,
                null, "application/json", null);
        HKResponse hkResponse = JSONObject.parseObject(responseStr, HKResponse.class);
        if (Objects.nonNull(hkResponse) && Objects.equals(HK_SUCCESS_STRING_CODE, hkResponse.getCode())) {
            String dataStr = String.valueOf(hkResponse.getData());
            List<SignalTimePO> signalTimePOs = JSONObject.parseObject(dataStr, new TypeReference<List<SignalTimePO>>(){});
            for (SignalTimePO signalTimePO : signalTimePOs) {
                Date date = new SimpleDateFormat(DateStyle.YYYY_MM_DD_HH_MM_SS.getValue()).parse(signalTimePO.getTime());
                return JsonViewObject.newInstance().success(date);
            }
        }
        return JsonViewObject.newInstance().fail("获取信号机校时失败");
    }
}
