package net.wanji.utc.service.systemadmin.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import net.wanji.databus.dao.mapper.ManufacturerInfoMapper;
import net.wanji.databus.po.ManufacturerInfoPO;
import net.wanji.utc.dto.systemadmin.DeleteByIntegerIdListDTO;
import net.wanji.utc.dto.systemadmin.ManufacturerInsertOrUpdateDTO;
import net.wanji.utc.dto.systemadmin.ManufacturerListDTO;
import net.wanji.utc.service.systemadmin.ManufacturerService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author Kent HAN
 * @date 2022/11/24 8:50
 */
@Service
public class ManufacturerServiceImpl implements ManufacturerService {
    @Autowired
    ManufacturerInfoMapper manufacturerInfoMapper;

    @Override
    public PageInfo<ManufacturerInfoPO> list(ManufacturerListDTO manufacturerListDTO) {
        Integer pageNum = manufacturerListDTO.getPageNum();
        Integer pageSize = manufacturerListDTO.getPageSize();
        String name = manufacturerListDTO.getName();

        PageHelper.startPage(pageNum, pageSize);
        List<ManufacturerInfoPO> manufacturerInfoPOList = manufacturerInfoMapper.selectByOptionalPartialName(name);
        PageInfo<ManufacturerInfoPO> borrowPOPageInfo = new PageInfo<>(manufacturerInfoPOList);

        return borrowPOPageInfo;
    }

    @Override
    @Transactional
    public void insertOrUpdate(ManufacturerInsertOrUpdateDTO inVO) {
        Integer id = inVO.getId();
        ManufacturerInfoPO manufacturerInfoPO = new ManufacturerInfoPO();
        BeanUtils.copyProperties(inVO, manufacturerInfoPO);
        if (id == null || id == 0) {
            // 添加
            manufacturerInfoMapper.insertOne(manufacturerInfoPO);
        } else {
            // 修改
            manufacturerInfoMapper.updateOne(manufacturerInfoPO);
        }
    }

    @Override
    @Transactional
    public void delete(DeleteByIntegerIdListDTO inVO) {
        List<Integer> ids = inVO.getIds();
        manufacturerInfoMapper.deleteBatch(ids);
    }
}
