package net.wanji.utc.task;

import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.ManufacturerApiInfoMapper;
import net.wanji.databus.dao.mapper.ManufacturerInfoMapper;
import net.wanji.databus.po.ManufacturerApiInfoPO;
import net.wanji.databus.po.ManufacturerInfoPO;
import net.wanji.utc.common.constant.Constants;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;

@Component
@Slf4j
@EnableAsync
public class ApiInfoTask {
    @Autowired
    ManufacturerApiInfoMapper manufacturerApiInfoMapper;
    @Autowired
    ManufacturerInfoMapper manufacturerInfoMapper;

    @Scheduled(fixedRate = 2 * 60 * 1000)
    public void apiInfoRefresh() {
        try {
            List<ManufacturerApiInfoPO> manufacturerApiInfoPOList = manufacturerApiInfoMapper.selectAll();
            if (manufacturerApiInfoPOList == null || manufacturerApiInfoPOList.isEmpty()) {
                return;
            }
            // 刷新JVM
            for (ManufacturerApiInfoPO infoPO : manufacturerApiInfoPOList) {
                Integer manufacturerId = infoPO.getManufacturerId();
                ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerId);
                String code = manufacturerInfoPO.getCode();
                String s = code + Constants.SEPARATOR_UNDER_LINE + infoPO.getCode();
                Constants.putManufacturerUrlMap(s, infoPO);
            }
            log.info("刷新" + manufacturerApiInfoPOList.size() + "条厂商URL至JVM");
        } catch (Exception e) {
            log.error("刷新厂商URL到JVM异常", e);
        }
    }
}